#include "ShaderConstants.fxh"
#include "util.fxh"


// SS10 Shader
// Made by @RyFol
// USE OF ANY CODE IS NOT PERMITTED

// http://www.ryfol.com/SS10



struct PS_Input
{
	float4 position : SV_Position;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef NEAR_WATER
	float cameraDist : TEXCOORD_2;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};



#define enableSSPEmapping
#define saturation 1.00  
#define exposure 1.300
#define brightness 1.500
#define gamma 0.900
#define contrast 1.97

float filmic_curve(float x) {

// Shoulder strength
float A = 0.22;
// Linear strength
float B = 0.5;
// Linear angle
float C = 0.15 * brightness;
// Toe strength
float D = 0.4 * gamma;
// Toe numerator
float E = 0.01 * contrast;
// Toe denominator
float F = 0.2;

return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;

}

float3 doSSPEmapping(float3 clr) {

float W = 1.0 / exposure;

#ifdef enableSSPEmapping

float Luma = dot(clr, float3(0.0, 0.3, 0.3));
float3 Chroma = clr - Luma;
clr = (Chroma * saturation) + Luma;

  clr = float3(filmic_curve(clr.r), filmic_curve(clr.g), filmic_curve(clr.b)) / filmic_curve(W);
#endif

return clr;
}

void main( in PS_Input PSInput, out PS_Output PSOutput )
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else


float torchlightaffects = (1.0 - PSInput.uv1.x);


float3 SSPEtorch = float3(0.900,0.200,-0.20);

#if !defined(TEXEL_AA) || !defined(TEXEL_AA_FEATURE) || (VERSION < 0xa000 /*D3D_FEATURE_LEVEL_10_0*/) 
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#else
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler1, PSInput.uv0 );
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
	PSInput.color.b = 1.0f;
#endif

#ifdef ALPHA_TEST
//If we know that all the verts in a triangle will have the same alpha, we should cull there first.
	#ifdef ALPHA_TO_COVERAGE
		float alphaThreshold = .05f;
	#else
		float alphaThreshold = .5f;
	#endif
	if(diffuse.a < alphaThreshold)
		discard;
#endif


#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);

#endif


#ifndef SEASONS

#if !defined(ALPHA_TEST) && !defined(BLEND)
	diffuse.a = PSInput.color.a;
#elif defined(BLEND)
#ifdef NEAR_WATER	
	diffuse.a *= PSInput.color.a;

	float alphaFadeOut = saturate(PSInput.cameraDist.x);
	diffuse.a = lerp(diffuse.a, 1.0f, alphaFadeOut);
#endif

#endif	
	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif


float s_amount = 0.0;
float nolight = (1.0 - PSInput.uv1.x);

if(PSInput.uv1.y < 0.875){s_amount = 0.05;}
if(PSInput.uv1.y < 0.874){s_amount = 0.10;}
if(PSInput.uv1.y < 0.873){s_amount = 0.15;}
if(PSInput.uv1.y < 0.872){s_amount = 0.20;}
if(PSInput.uv1.y < 0.871){s_amount = 0.25;}
if(PSInput.uv1.y < 0.870){s_amount = 0.30;}
if(PSInput.uv1.y < 0.869){s_amount = 0.35;}
if(PSInput.uv1.y < 0.868){s_amount = 0.40;}
if(PSInput.uv1.y < 0.867){s_amount = 0.45;}

float3 SSPEshadow = float3(0.0,0.0,0.0);

diffuse.rgb = lerp(diffuse.rgb, SSPEshadow.rgb, s_amount * nolight * torchlightaffects);



#ifdef FOG
	diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif


diffuse.rgb = doSSPEmapping(diffuse.rgb);
diffuse.rgb += SSPEtorch.rgb*max(0.0,PSInput.uv1.x-0.5f);

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}